@@ -7,7 +7,7 @@
 -- Module      :  $Header$
 -- Description :  Data.IntMap with Enum keys.
 -- Copyright   :  (c) 2011-2019 Michal Terepeta
---                (c) 2019-2020 Mikolaj Konarski and others (see git history)
+--                (c) 2019-2022 Mikolaj Konarski and others (see git history)
 -- License     :  BSD3
 -- Maintainer  :  mikolaj.konarski@funktory.com
 -- Stability   :  alpha
@@ -54,6 +54,7 @@ module Data.EnumMap.Base
   , updateWithKey
   , updateLookupWithKey
   , alter
+  , alterF
 
   -- * Combine
 
@@ -146,6 +147,8 @@ module Data.EnumMap.Base
   , isProperSubmapOfBy
 
   -- * Min\/Max
+  , lookupMin
+  , lookupMax
   , findMin
   , findMax
   , deleteMin
@@ -180,7 +183,7 @@ import Data.Monoid ( Monoid )
 import Data.Semigroup ( Semigroup )
 import Data.Traversable ( Traversable )
 import Data.Typeable ( Typeable )
-
+import Data.Aeson ( FromJSON(..), ToJSON(..) )
 import Text.Read
 
 -- | Wrapper for 'IntMap' with 'Enum' keys.
@@ -195,8 +198,14 @@ instance (Enum k, Show k, Show a) => Show (EnumMap k a) where
 instance (Enum k, Read k, Read a) => Read (EnumMap k a) where
   readPrec = parens . prec 10 $ do
     Ident "fromList" <- lexP
-    list <- readPrec
-    return (fromList list)
+    fromList <$> readPrec
+
+instance (ToJSON a) => ToJSON (EnumMap k a) where
+    toJSON = toJSON . unWrap
+    toEncoding = toEncoding . unWrap
+
+instance (FromJSON a) => FromJSON (EnumMap k a) where
+    parseJSON = fmap (EnumMap . I.fromList) . parseJSON
 
 --
 -- Conversion to/from 'IntMap'.
@@ -321,6 +330,10 @@ alter :: (Enum k) => (Maybe a -> Maybe a) -> k -> EnumMap k a -> EnumMap k a
 alter f k = EnumMap . I.alter f (fromEnum k) . unWrap
 {-# INLINE alter #-}
 
+alterF :: (Enum k, Functor f) => (Maybe a -> f (Maybe a)) -> k -> EnumMap k a -> f (EnumMap k a)
+alterF f k = fmap EnumMap . I.alterF f (fromEnum k) . unWrap
+{-# INLINE alterF #-}
+
 unions :: [EnumMap k a] -> EnumMap k a
 unions = EnumMap . I.unions . P.map unWrap
 {-# INLINE unions #-}
@@ -432,6 +445,14 @@ deleteFindMin :: (Enum k) => EnumMap k a -> ((k, a), EnumMap k a)
 deleteFindMin = (first toEnum *** EnumMap) . I.deleteFindMin . unWrap
 {-# INLINE deleteFindMin #-}
 
+lookupMin :: (Enum k) => EnumMap k a -> Maybe (k, a)
+lookupMin = fmap (first toEnum) . I.lookupMin . unWrap
+{-# INLINE lookupMin #-}
+
+lookupMax :: (Enum k) => EnumMap k a -> Maybe (k, a)
+lookupMax = fmap (first toEnum) . I.lookupMax . unWrap
+{-# INLINE lookupMax #-}
+
 findMin :: (Enum k) => EnumMap k a -> (k, a)
 findMin = first toEnum . I.findMin . unWrap
 {-# INLINE findMin #-}
